@@ -1,7 +1,6 @@
-from __future__ import absolute_import
+__version__ = '5.0.0'
 
-
-from .cal import (
+from icalendar.cal import (
     Calendar,
     Event,
     Todo,
@@ -14,7 +13,7 @@ from .cal import (
     ComponentFactory,
 )
 # Property Data Value Types
-from .prop import (
+from icalendar.prop import (
     vBinary,
     vBoolean,
     vCalAddress,
@@ -36,27 +35,14 @@ from .prop import (
     TypesFactory,
 )
 # useful tzinfo subclasses
-from .prop import (
+from icalendar.prop import (
     FixedOffset,
     LocalTimezone,
 )
 # Parameters and helper methods for splitting and joining string with escaped
 # chars.
-from .parser import (
+from icalendar.parser import (
     Parameters,
     q_split,
     q_join,
 )
-
-
-__all__ = [
-    Calendar, Event, Todo, Journal,
-    FreeBusy, Alarm, ComponentFactory,
-    Timezone, TimezoneStandard, TimezoneDaylight,
-    vBinary, vBoolean, vCalAddress, vDatetime, vDate,
-    vDDDTypes, vDuration, vFloat, vInt, vPeriod,
-    vWeekday, vFrequency, vRecur, vText, vTime, vUri,
-    vGeo, vUTCOffset, TypesFactory,
-    FixedOffset, LocalTimezone,
-    Parameters, q_split, q_join,
-]
