@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """This module contains the parser/generators (or coders/encoders if you
 prefer) for the classes/datatypes that are used in iCalendar:
 
@@ -18,54 +17,60 @@ prefer) for the classes/datatypes that are used in iCalendar:
 
 ###########################################################################
 
-iCalendar properties has values. The values are strongly typed. This module
-defines these types, calling val.to_ical() on them, Will render them as defined
+iCalendar properties have values. The values are strongly typed. This module
+defines these types, calling val.to_ical() on them will render them as defined
 in rfc2445.
 
 If you pass any of these classes a Python primitive, you will have an object
 that can render itself as iCalendar formatted date.
 
-Property Value Data Types starts with a 'v'. they all have an to_ical() and
+Property Value Data Types start with a 'v'. they all have an to_ical() and
 from_ical() method. The to_ical() method generates a text string in the
 iCalendar format. The from_ical() method can parse this format and return a
-primitive Python datatype. So it should allways be true that:
+primitive Python datatype. So it should always be true that:
 
     x == vDataType.from_ical(VDataType(x).to_ical())
 
 These types are mainly used for parsing and file generation. But you can set
 them directly.
-
 """
-from __future__ import absolute_import
-import re
-import pytz
+from datetime import date
+from datetime import datetime
+from datetime import time
+from datetime import timedelta
+from datetime import tzinfo
+
+try:
+    from dateutil.tz import tzutc
+except ImportError:
+    tzutc = None
+
+from icalendar.caselessdict import CaselessDict
+from icalendar.parser import Parameters
+from icalendar.parser import escape_char
+from icalendar.parser import tzid_from_dt
+from icalendar.parser import unescape_char
+from icalendar.parser_tools import DEFAULT_ENCODING
+from icalendar.parser_tools import SEQUENCE_TYPES
+from icalendar.parser_tools import to_unicode
+from icalendar.timezone_cache import _timezone_cache
+from icalendar.windows_to_olson import WINDOWS_TO_OLSON
+
+import base64
 import binascii
+import pytz
+import re
 import time as _time
-from datetime import (
-    datetime,
-    timedelta,
-    time,
-    date,
-    tzinfo,
-)
-from dateutil.tz import tzutc
-from .parser_tools import SEQUENCE_TYPES, DEFAULT_ENCODING, to_unicode
-from .caselessdict import CaselessDict
-from .parser import (
-    Parameters,
-    escape_char,
-    unescape_char,
-    tzid_from_dt,
-)
+
 
 DATE_PART = r'(\d+)D'
 TIME_PART = r'T(?:(\d+)H)?(?:(\d+)M)?(?:(\d+)S)?'
-DATETIME_PART = '(?:%s)?(?:%s)?' % (DATE_PART, TIME_PART)
+DATETIME_PART = f'(?:{DATE_PART})?(?:{TIME_PART})?'
 WEEKS_PART = r'(\d+)W'
 DURATION_REGEX = re.compile(r'([-+]?)P(?:%s|%s)$'
                             % (WEEKS_PART, DATETIME_PART))
-WEEKDAY_RULE = re.compile('(?P<signal>[+-]?)(?P<relative>[\d]?)'
-                          '(?P<weekday>[\w]{2})$')
+WEEKDAY_RULE = re.compile(r'(?P<signal>[+-]?)(?P<relative>[\d]?)'
+                          r'(?P<weekday>[\w]{2})$')
 
 
 ####################################################
@@ -126,60 +131,57 @@ class LocalTimezone(tzinfo):
         return tt.tm_isdst > 0
 
 
-class vBinary(object):
+class vBinary:
     """Binary property values are base 64 encoded.
-
     """
 
     def __init__(self, obj):
-        self.obj = obj
+        self.obj = to_unicode(obj)
         self.params = Parameters(encoding='BASE64', value="BINARY")
 
     def __repr__(self):
         return "vBinary('%s')" % self.to_ical()
 
     def to_ical(self):
-        return binascii.b2a_base64(self.obj)[:-1]
+        return binascii.b2a_base64(self.obj.encode('utf-8'))[:-1]
 
     @staticmethod
     def from_ical(ical):
         try:
-            return ical.decode('base-64')
+            return base64.b64decode(ical)
         except UnicodeError:
             raise ValueError('Not valid base 64 encoding.')
 
 
 class vBoolean(int):
     """Returns specific string according to state.
-
     """
-    BOOL_MAP = CaselessDict(true=True, false=False)
+    BOOL_MAP = CaselessDict({'true': True, 'false': False})
 
     def __new__(cls, *args, **kwargs):
-        self = super(vBoolean, cls).__new__(cls, *args, **kwargs)
+        self = super().__new__(cls, *args, **kwargs)
         self.params = Parameters()
         return self
 
     def to_ical(self):
         if self:
-            return 'TRUE'
-        return 'FALSE'
+            return b'TRUE'
+        return b'FALSE'
 
     @classmethod
     def from_ical(cls, ical):
         try:
             return cls.BOOL_MAP[ical]
-        except:
+        except Exception:
             raise ValueError("Expected 'TRUE' or 'FALSE'. Got %s" % ical)
 
 
-class vCalAddress(unicode):
+class vCalAddress(str):
     """This just returns an unquoted string.
-
     """
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vCalAddress, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         self.params = Parameters()
         return self
 
@@ -196,47 +198,44 @@ class vCalAddress(unicode):
 
 class vFloat(float):
     """Just a float.
-
     """
     def __new__(cls, *args, **kwargs):
-        self = super(vFloat, cls).__new__(cls, *args, **kwargs)
+        self = super().__new__(cls, *args, **kwargs)
         self.params = Parameters()
         return self
 
     def to_ical(self):
-        return str(self)
+        return str(self).encode('utf-8')
 
     @classmethod
     def from_ical(cls, ical):
         try:
             return cls(ical)
-        except:
+        except Exception:
             raise ValueError('Expected float value, got: %s' % ical)
 
 
 class vInt(int):
     """Just an int.
-
     """
     def __new__(cls, *args, **kwargs):
-        self = super(vInt, cls).__new__(cls, *args, **kwargs)
+        self = super().__new__(cls, *args, **kwargs)
         self.params = Parameters()
         return self
 
     def to_ical(self):
-        return str(self)
+        return str(self).encode('utf-8')
 
     @classmethod
     def from_ical(cls, ical):
         try:
             return cls(ical)
-        except:
+        except Exception:
             raise ValueError('Expected int, got: %s' % ical)
 
 
-class vDDDLists(object):
+class vDDDLists:
     """A list of vDDDTypes values.
-
     """
     def __init__(self, dt_list):
         if not hasattr(dt_list, '__iter__'):
@@ -256,7 +255,7 @@ class vDDDLists(object):
 
     def to_ical(self):
         dts_ical = (dt.to_ical() for dt in self.dts)
-        return ",".join(dts_ical)
+        return b",".join(dts_ical)
 
     @staticmethod
     def from_ical(ical, timezone=None):
@@ -266,31 +265,47 @@ class vDDDLists(object):
             out.append(vDDDTypes.from_ical(ical_dt, timezone=timezone))
         return out
 
+class vCategory:
 
-class vDDDTypes(object):
+    def __init__(self, c_list):
+        if not hasattr(c_list, '__iter__'):
+            c_list = [c_list]
+        self.cats = [vText(c) for c in c_list]
+
+    def to_ical(self):
+        return b",".join([c.to_ical() for c in self.cats])
+
+    @staticmethod
+    def from_ical(ical):
+        ical = to_unicode(ical)
+        out = unescape_char(ical).split(',')
+        return out
+
+
+class vDDDTypes:
     """A combined Datetime, Date or Duration parser/generator. Their format
     cannot be confused, and often values can be of either types.
     So this is practical.
-
     """
     def __init__(self, dt):
-        if type(dt) not in (datetime, date, timedelta, time):
-            raise ValueError('You must use datetime, date, timedelta or time')
+        if not isinstance(dt, (datetime, date, timedelta, time, tuple)):
+            raise ValueError('You must use datetime, date, timedelta, '
+                             'time or tuple (for periods)')
         if isinstance(dt, datetime):
-            self.params = Parameters(dict(value='DATE-TIME'))
+            self.params = Parameters({'value': 'DATE-TIME'})
         elif isinstance(dt, date):
-            self.params = Parameters(dict(value='DATE'))
+            self.params = Parameters({'value': 'DATE'})
         elif isinstance(dt, time):
-            self.params = Parameters(dict(value='TIME'))
+            self.params = Parameters({'value': 'TIME'})
+        elif isinstance(dt, tuple):
+            self.params = Parameters({'value': 'PERIOD'})
 
         if (isinstance(dt, datetime) or isinstance(dt, time))\
-            and getattr(dt, 'tzinfo', False):
+                and getattr(dt, 'tzinfo', False):
             tzinfo = dt.tzinfo
-            if tzinfo is not pytz.utc and not isinstance(tzinfo, tzutc):
-                # set the timezone as a parameter to the property
-                tzid = tzid_from_dt(dt)
-                if tzid:
-                    self.params.update({'TZID': tzid})
+            tzid = tzid_from_dt(dt)
+            if tzid != 'UTC':
+                self.params.update({'TZID': tzid})
         self.dt = dt
 
     def to_ical(self):
@@ -303,37 +318,50 @@ class vDDDTypes(object):
             return vDuration(dt).to_ical()
         elif isinstance(dt, time):
             return vTime(dt).to_ical()
+        elif isinstance(dt, tuple) and len(dt) == 2:
+            return vPeriod(dt).to_ical()
         else:
-            raise ValueError('Unknown date type')
+            raise ValueError(f'Unknown date type: {type(dt)}')
+
+    def __eq__(self, other):
+        if isinstance(other, vDDDTypes):
+            return self.params == other.params
+        return False
 
     @classmethod
     def from_ical(cls, ical, timezone=None):
         if isinstance(ical, cls):
             return ical.dt
         u = ical.upper()
-        if u.startswith('-P') or u.startswith('P'):
+        if u.startswith(('P', '-P', '+P')):
             return vDuration.from_ical(ical)
-        try:
+        if '/' in u:
+            return vPeriod.from_ical(ical)
+
+        if len(ical) in (15, 16):
             return vDatetime.from_ical(ical, timezone=timezone)
-        except ValueError:
-            try:
-                return vDate.from_ical(ical)
-            except ValueError:
-                return vTime.from_ical(ical)
+        elif len(ical) == 8:
+            return vDate.from_ical(ical)
+        elif len(ical) in (6, 7):
+            return vTime.from_ical(ical)
+        else:
+            raise ValueError(
+                "Expected datetime, date, or time, got: '%s'" % ical
+            )
 
 
-class vDate(object):
+class vDate:
     """Render and generates iCalendar date format.
-
     """
     def __init__(self, dt):
         if not isinstance(dt, date):
             raise ValueError('Value MUST be a date instance')
         self.dt = dt
-        self.params = Parameters(dict(value='DATE'))
+        self.params = Parameters({'value': 'DATE'})
 
     def to_ical(self):
-        return "%04d%02d%02d" % (self.dt.year, self.dt.month, self.dt.day)
+        s = "%04d%02d%02d" % (self.dt.year, self.dt.month, self.dt.day)
+        return s.encode('utf-8')
 
     @staticmethod
     def from_ical(ical):
@@ -344,11 +372,11 @@ class vDate(object):
                 int(ical[6:8]),  # day
             )
             return date(*timetuple)
-        except:
+        except Exception:
             raise ValueError('Wrong date format %s' % ical)
 
 
-class vDatetime(object):
+class vDatetime:
     """Render and generates icalendar datetime format.
 
     vDatetime is timezone aware and uses the pytz library, an implementation of
@@ -358,7 +386,6 @@ class vDatetime(object):
     tzinfo component, if present. Otherwise an timezone-naive object is
     created. Be aware that there are certain limitations with timezone naive
     DATE-TIME components in the icalendar standard.
-
     """
     def __init__(self, dt):
         self.dt = dt
@@ -380,7 +407,7 @@ class vDatetime(object):
             s += "Z"
         elif tzid:
             self.params.update({'TZID': tzid})
-        return s
+        return s.encode('utf-8')
 
     @staticmethod
     def from_ical(ical, timezone=None):
@@ -389,7 +416,10 @@ class vDatetime(object):
             try:
                 tzinfo = pytz.timezone(timezone)
             except pytz.UnknownTimeZoneError:
-                pass
+                if timezone in WINDOWS_TO_OLSON:
+                    tzinfo = pytz.timezone(WINDOWS_TO_OLSON.get(timezone))
+                else:
+                    tzinfo = _timezone_cache.get(timezone, None)
 
         try:
             timetuple = (
@@ -405,17 +435,16 @@ class vDatetime(object):
             elif not ical[15:]:
                 return datetime(*timetuple)
             elif ical[15:16] == 'Z':
-                return datetime(tzinfo=pytz.utc, *timetuple)
+                return pytz.utc.localize(datetime(*timetuple))
             else:
                 raise ValueError(ical)
-        except:
+        except Exception:
             raise ValueError('Wrong datetime format: %s' % ical)
 
 
-class vDuration(object):
+class vDuration:
     """Subclass of timedelta that renders itself in the iCalendar DURATION
     format.
-
     """
 
     def __init__(self, td):
@@ -426,25 +455,29 @@ class vDuration(object):
 
     def to_ical(self):
         sign = ""
-        if self.td.days < 0:
+        td = self.td
+        if td.days < 0:
             sign = "-"
-            self.td = -self.td
+            td = -td
         timepart = ""
-        if self.td.seconds:
+        if td.seconds:
             timepart = "T"
-            hours = self.td.seconds // 3600
-            minutes = self.td.seconds % 3600 // 60
-            seconds = self.td.seconds % 60
+            hours = td.seconds // 3600
+            minutes = td.seconds % 3600 // 60
+            seconds = td.seconds % 60
             if hours:
                 timepart += "%dH" % hours
             if minutes or (hours and seconds):
                 timepart += "%dM" % minutes
             if seconds:
                 timepart += "%dS" % seconds
-        if self.td.days == 0 and timepart:
-            return "%sP%s" % (sign, timepart)
+        if td.days == 0 and timepart:
+            return (str(sign).encode('utf-8') + b'P' +
+                    str(timepart).encode('utf-8'))
         else:
-            return "%sP%dD%s" % (sign, abs(self.td.days), timepart)
+            return (str(sign).encode('utf-8') + b'P' +
+                    str(abs(td.days)).encode('utf-8') +
+                    b'D' + str(timepart).encode('utf-8'))
 
     @staticmethod
     def from_ical(ical):
@@ -461,13 +494,12 @@ class vDuration(object):
             if sign == '-':
                 value = -value
             return value
-        except:
+        except Exception:
             raise ValueError('Invalid iCalendar duration: %s' % ical)
 
 
-class vPeriod(object):
+class vPeriod:
     """A precise period of time.
-
     """
     def __init__(self, per):
         start, end_or_duration = per
@@ -515,10 +547,10 @@ class vPeriod(object):
 
     def to_ical(self):
         if self.by_duration:
-            return '%s/%s' % (vDatetime(self.start).to_ical(),
-                              vDuration(self.duration).to_ical())
-        return '%s/%s' % (vDatetime(self.start).to_ical(),
-                          vDatetime(self.end).to_ical())
+            return (vDatetime(self.start).to_ical() + b'/' +
+                    vDuration(self.duration).to_ical())
+        return (vDatetime(self.start).to_ical() + b'/' +
+                vDatetime(self.end).to_ical())
 
     @staticmethod
     def from_ical(ical):
@@ -527,7 +559,7 @@ class vPeriod(object):
             start = vDDDTypes.from_ical(start)
             end_or_duration = vDDDTypes.from_ical(end_or_duration)
             return (start, end_or_duration)
-        except:
+        except Exception:
             raise ValueError('Expected period format, got: %s' % ical)
 
     def __repr__(self):
@@ -535,12 +567,11 @@ class vPeriod(object):
             p = (self.start, self.duration)
         else:
             p = (self.start, self.end)
-        return 'vPeriod(%r)' % p
+        return f'vPeriod({p!r})'
 
 
-class vWeekday(unicode):
+class vWeekday(str):
     """This returns an unquoted weekday abbrevation.
-
     """
     week_days = CaselessDict({
         "SU": 0, "MO": 1, "TU": 2, "WE": 3, "TH": 4, "FR": 5, "SA": 6,
@@ -548,7 +579,7 @@ class vWeekday(unicode):
 
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vWeekday, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         match = WEEKDAY_RULE.match(self)
         if match is None:
             raise ValueError('Expected weekday abbrevation, got: %s' % self)
@@ -556,7 +587,7 @@ class vWeekday(unicode):
         sign = match['signal']
         weekday = match['weekday']
         relative = match['relative']
-        if not weekday in vWeekday.week_days or sign not in '+-':
+        if weekday not in vWeekday.week_days or sign not in '+-':
             raise ValueError('Expected weekday abbrevation, got: %s' % self)
         self.relative = relative and int(relative) or None
         self.params = Parameters()
@@ -569,13 +600,12 @@ class vWeekday(unicode):
     def from_ical(cls, ical):
         try:
             return cls(ical.upper())
-        except:
+        except Exception:
             raise ValueError('Expected weekday abbrevation, got: %s' % ical)
 
 
-class vFrequency(unicode):
+class vFrequency(str):
     """A simple class that catches illegal values.
-
     """
 
     frequencies = CaselessDict({
@@ -590,8 +620,8 @@ class vFrequency(unicode):
 
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vFrequency, cls).__new__(cls, value)
-        if not self in vFrequency.frequencies:
+        self = super().__new__(cls, value)
+        if self not in vFrequency.frequencies:
             raise ValueError('Expected frequency, got: %s' % self)
         self.params = Parameters()
         return self
@@ -603,13 +633,12 @@ class vFrequency(unicode):
     def from_ical(cls, ical):
         try:
             return cls(ical.upper())
-        except:
+        except Exception:
             raise ValueError('Expected frequency, got: %s' % ical)
 
 
 class vRecur(CaselessDict):
     """Recurrence definition.
-
     """
 
     frequencies = ["SECONDLY", "MINUTELY", "HOURLY", "DAILY", "WEEKLY",
@@ -628,6 +657,7 @@ class vRecur(CaselessDict):
         'BYSECOND': vInt,
         'BYMINUTE': vInt,
         'BYHOUR': vInt,
+        'BYWEEKNO': vInt,
         'BYMONTHDAY': vInt,
         'BYYEARDAY': vInt,
         'BYMONTH': vInt,
@@ -639,18 +669,22 @@ class vRecur(CaselessDict):
     })
 
     def __init__(self, *args, **kwargs):
-        CaselessDict.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.params = Parameters()
 
     def to_ical(self):
         result = []
         for key, vals in self.sorted_items():
-            typ = self.types[key]
+            typ = self.types.get(key, vText)
             if not isinstance(vals, SEQUENCE_TYPES):
                 vals = [vals]
-            vals = ','.join(typ(val).to_ical() for val in vals)
-            result.append('%s=%s' % (key, vals))
-        return ';'.join(result)
+            vals = b','.join(typ(val).to_ical() for val in vals)
+
+            # CaselessDict keys are always unicode
+            key = key.encode(DEFAULT_ENCODING)
+            result.append(key + b'=' + vals)
+
+        return b';'.join(result)
 
     @classmethod
     def parse_type(cls, key, values):
@@ -665,21 +699,25 @@ class vRecur(CaselessDict):
         try:
             recur = cls()
             for pairs in ical.split(';'):
-                key, vals = pairs.split('=')
+                try:
+                    key, vals = pairs.split('=')
+                except ValueError:
+                    # E.g. incorrect trailing semicolon, like (issue #157):
+                    # FREQ=YEARLY;BYMONTH=11;BYDAY=1SU;
+                    continue
                 recur[key] = cls.parse_type(key, vals)
             return dict(recur)
-        except:
+        except Exception:
             raise ValueError('Error in recurrence rule: %s' % ical)
 
 
-class vText(unicode):
+class vText(str):
     """Simple text.
-
     """
 
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vText, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         self.encoding = encoding
         self.params = Parameters()
         return self
@@ -696,9 +734,8 @@ class vText(unicode):
         return cls(ical_unesc)
 
 
-class vTime(object):
+class vTime:
     """Render and generates iCalendar time format.
-
     """
 
     def __init__(self, *args):
@@ -708,7 +745,7 @@ class vTime(object):
             self.dt = args[0]
         else:
             self.dt = time(*args)
-        self.params = Parameters(dict(value='TIME'))
+        self.params = Parameters({'value': 'TIME'})
 
     def to_ical(self):
         return self.dt.strftime("%H%M%S")
@@ -719,18 +756,17 @@ class vTime(object):
         try:
             timetuple = (int(ical[:2]), int(ical[2:4]), int(ical[4:6]))
             return time(*timetuple)
-        except:
+        except Exception:
             raise ValueError('Expected time, got: %s' % ical)
 
 
-class vUri(unicode):
+class vUri(str):
     """Uniform resource identifier is basically just an unquoted string.
-
     """
 
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vUri, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         self.params = Parameters()
         return self
 
@@ -741,13 +777,12 @@ class vUri(unicode):
     def from_ical(cls, ical):
         try:
             return cls(ical)
-        except:
+        except Exception:
             raise ValueError('Expected , got: %s' % ical)
 
 
-class vGeo(object):
+class vGeo:
     """A special type that is only indirectly defined in the rfc.
-
     """
 
     def __init__(self, geo):
@@ -755,7 +790,7 @@ class vGeo(object):
             latitude, longitude = (geo[0], geo[1])
             latitude = float(latitude)
             longitude = float(longitude)
-        except:
+        except Exception:
             raise ValueError('Input must be (float, float) for '
                              'latitude and longitude')
         self.latitude = latitude
@@ -763,22 +798,26 @@ class vGeo(object):
         self.params = Parameters()
 
     def to_ical(self):
-        return '%s;%s' % (self.latitude, self.longitude)
+        return f'{self.latitude};{self.longitude}'
 
     @staticmethod
     def from_ical(ical):
         try:
             latitude, longitude = ical.split(';')
             return (float(latitude), float(longitude))
-        except:
+        except Exception:
             raise ValueError("Expected 'float;float' , got: %s" % ical)
 
 
-class vUTCOffset(object):
+class vUTCOffset:
     """Renders itself as a utc offset.
-
     """
 
+    ignore_exceptions = False   # if True, and we cannot parse this
+                                # component, we will silently ignore
+                                # it, rather than let the exception
+                                # propagate upwards
+
     def __init__(self, td):
         if not isinstance(td, timedelta):
             raise ValueError('Offset value MUST be a timedelta instance')
@@ -787,9 +826,9 @@ class vUTCOffset(object):
 
     def to_ical(self):
 
-        if self.td<timedelta(0):
+        if self.td < timedelta(0):
             sign = '-%s'
-            td = timedelta(0)-self.td # get timedelta relative to 0
+            td = timedelta(0) - self.td  # get timedelta relative to 0
         else:
             # Google Calendar rejects '0000' but accepts '+0000'
             sign = '+%s'
@@ -816,9 +855,9 @@ class vUTCOffset(object):
                                              int(ical[3:5]),
                                              int(ical[5:7] or 0))
             offset = timedelta(hours=hours, minutes=minutes, seconds=seconds)
-        except:
+        except Exception:
             raise ValueError('Expected utc offset, got: %s' % ical)
-        if offset >= timedelta(hours=24):
+        if not cls.ignore_exceptions and offset >= timedelta(hours=24):
             raise ValueError(
                 'Offset must be less than 24 hours, was %s' % ical)
         if sign == '-':
@@ -826,15 +865,14 @@ class vUTCOffset(object):
         return offset
 
 
-class vInline(unicode):
+class vInline(str):
     """This is an especially dumb class that just holds raw unparsed text and
     has parameters. Conversion of inline values are handled by the Component
     class, so no further processing is needed.
-
     """
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vInline, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         self.params = Parameters()
         return self
 
@@ -852,13 +890,12 @@ class TypesFactory(CaselessDict):
 
     The value and parameter names don't overlap. So one factory is enough for
     both kinds.
-
     """
 
     def __init__(self, *args, **kwargs):
         "Set keys to upper for initial dict"
-        CaselessDict.__init__(self, *args, **kwargs)
-        self.all_types = [
+        super().__init__(*args, **kwargs)
+        self.all_types = (
             vBinary,
             vBoolean,
             vCalAddress,
@@ -879,7 +916,8 @@ class TypesFactory(CaselessDict):
             vUTCOffset,
             vUri,
             vWeekday,
-        ]
+            vCategory,
+        )
         self['binary'] = vBinary
         self['boolean'] = vBoolean
         self['cal-address'] = vCalAddress
@@ -897,6 +935,7 @@ class TypesFactory(CaselessDict):
         self['geo'] = vGeo
         self['inline'] = vInline
         self['date-time-list'] = vDDDLists
+        self['categories'] = vCategory
 
     #################################################
     # Property types
@@ -912,7 +951,7 @@ class TypesFactory(CaselessDict):
         'version': 'text',
         # Descriptive Component Properties
         'attach': 'uri',
-        'categories': 'text',
+        'categories': 'categories',
         'class': 'text',
         'comment': 'text',
         'description': 'text',
