@@ -1,10 +1,8 @@
-import icalendar
-import datetime
-import os
-
 import unittest
-if not hasattr(unittest.TestCase, 'assertIsNotNone'):
-    import unittest2 as unittest
+
+import datetime
+import icalendar
+import os
 
 
 class TestTime(unittest.TestCase):
@@ -17,7 +15,7 @@ class TestTime(unittest.TestCase):
 
     def test_create_from_ical(self):
         directory = os.path.dirname(__file__)
-        ics = open(os.path.join(directory, 'time.ics'),'rb')
+        ics = open(os.path.join(directory, 'time.ics'), 'rb')
         cal = icalendar.Calendar.from_ical(ics.read())
         ics.close()
 
@@ -27,5 +25,5 @@ class TestTime(unittest.TestCase):
     def test_create_to_ical(self):
         cal = icalendar.Calendar()
         cal.add('X-SOMETIME', datetime.time(17, 20, 10))
-        self.assertTrue('X-SOMETIME;VALUE=TIME:172010' in
+        self.assertTrue(b'X-SOMETIME;VALUE=TIME:172010' in
                         cal.to_ical().splitlines())
